/*
 * Copyright (c) 2009, Stefan Krah
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>


#include "decimal.h"



/* Nonsense version of escape-time algorithm for calculating a mandelbrot
 * set. Just for benchmarking. */
void
color_point(Decimal64 *x0, Decimal64 *y0, int maxiter)
{
	Decimal64 x, y, x2, y2, two, four;
	int i = 0;

  	__bid64_from_string(&two, "2");
  	__bid64_from_string(&four, "4");

	__bid64_copy(&x, x0);
	__bid64_copy(&y, y0);
	__bid64_mul(&x2, &x, &x);
	__bid64_mul(&y2, &y, &y);

	for (i = 0; i < maxiter; i++) {
		__bid64_mul(&y, &x, &y);
		__bid64_mul(&y, &y, &two);
		__bid64_add(&y, &y, y0);
		__bid64_sub(&x, &x2, &y2);
		__bid64_add(&x, &x, x0);
		__bid64_mul(&x2, &x, &x);
		__bid64_mul(&y2, &y, &y);
	}

	__bid64_copy(x0, &x);
}


int
main(int argc, char **argv)
{
	Decimal64 x0, y0;
	long iter = 10000000;
	clock_t start_clock, end_clock;
	char s[100];

	if (argc != 2) {
		fprintf(stderr, "usage: bench iter\n");
		exit(1);
	}
	iter = strtol(argv[1], NULL, 10);

	__bid_IDEC_glbround = _IDEC_nearesteven;
	__bid_IDEC_glbflags = _IDEC_allflagsclear;

	__bid64_from_string(&x0, "0.222");
	__bid64_from_string(&y0, "0.333");
	__bid64_to_string(s, &x0);


	start_clock = clock();
	color_point(&x0, &y0, iter);
	end_clock = clock();

	printf("time: %f\n\n", (double)(end_clock-start_clock)/(double)CLOCKS_PER_SEC);

	__bid64_to_string(s, &x0);
	printf("%s\n", s);

	return 0;
}









