/*
 * Copyright (c) 2009, Stefan Krah
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */


#include <stdio.h>
#include <stdlib.h>
#include <time.h>


#define  DECNUMDIGITS 100
#include "decNumber.h"


/* Nonsense version of escape-time algorithm for calculating a mandelbrot
 * set. Just for benchmarking. */
void
color_point(decNumber *x0, decNumber *y0, int maxiter, decContext *ctx)
{
	decNumber x, y, sq_x, sq_y, two, four;
	int i;
	
  	decNumberFromString(&two, "2", ctx);
  	decNumberFromString(&four, "4", ctx);

	decNumberCopy(&x, x0);
	decNumberCopy(&y, y0);
	decNumberMultiply(&sq_x, &x, &x, ctx);
	decNumberMultiply(&sq_y, &y, &y, ctx);

	for (i = 0; i < maxiter; i++) {
		decNumberMultiply(&y, &x, &y, ctx);
		decNumberMultiply(&y, &y, &two, ctx);
		decNumberAdd(&y, &y, y0, ctx);
		decNumberSubtract(&x, &sq_x, &sq_y, ctx);
		decNumberAdd(&x, &x, x0, ctx);
		decNumberMultiply(&sq_x, &x, &x, ctx);
		decNumberMultiply(&sq_y, &y, &y, ctx);
	}

	decNumberCopy(x0, &x);
}


int
main(int argc, char **argv)
{
	decContext ctx;
	decNumber x0, y0;
	uint32_t prec = 19;
	long iter = 10000000;
	clock_t start_clock, end_clock;
	char s[DECNUMDIGITS+14];

	if (argc != 3) {
		fprintf(stderr, "usage: bench prec iter\n");
		exit(1);
	}
	prec = strtoul(argv[1], NULL, 10);
	iter = strtol(argv[2], NULL, 10);

	decContextDefault(&ctx, DEC_INIT_BASE);
	ctx.digits = prec;

	decNumberFromString(&x0, "0.222", &ctx);
	decNumberFromString(&y0, "0.333", &ctx);

	start_clock = clock();
	color_point(&x0, &y0, iter, &ctx);
	end_clock = clock();

	printf("time: %f\n\n", (double)(end_clock-start_clock)/(double)CLOCKS_PER_SEC);

	decNumberToString(&x0, s);
	printf("%s\n", s);

	return 0;
}


